# -*- coding: utf-8 -*-

# Copyright (C) 2011, 2014 kaliko <kaliko@azylum.org>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 only.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import threading
import time
import traceback

from feedparser import parse as feed_parse

from .plugin import Plugin, botcmd


html_escape_table = {
        "&": "&amp;",
        '"': "&quot;",
        "'": "&apos;",
        ">": "&gt;",
        "<": "&lt;",
        }


def html_escape(text):
    """Produce entities within text."""
    return ''.join(html_escape_table.get(c, c) for c in text)


def strtm_to_dtm(struc_time):
    return datetime.datetime(*struc_time[:6])


class FeedMonitor(threading.Thread):
    def __init__(self, plugin):
        threading.Thread.__init__(self)
        self.feeds_list = plugin.FEEDS
        self.tempo = plugin.TEMPO
        self.plugin = plugin
        self.last_check = datetime.datetime.utcnow()
        self.seen = dict()
        self.thread_killed = False

    def new_posts(self, feed):
        """Send new posts in feed"""
        parsed_feed = feed_parse(feed)

        # Cannot resolve address
        if 'status' not in parsed_feed:
            self.plugin.log.error('Error from "%s": %s.' %
                    (feed, parsed_feed.bozo_exception.__repr__()))
            return

        # unusual return http code
        if parsed_feed.status != 200:
            self.plugin.log.error(
                'Got code %(status)d from "%(href)s" (please update).' %
                    parsed_feed)
            return

        feed_updated = parsed_feed.feed.get('updated_parsed', None)

        # Avoid looping over all posts if possible
        if feed_updated and strtm_to_dtm(feed_updated) < self.last_check:
            self.plugin.log.debug('updated   : %s' % strtm_to_dtm(feed_updated))
            self.plugin.log.debug('last check: %s' % self.last_check)
            return

        title = '"%s":' % parsed_feed.feed.get('title', 'n/a')
        xtitle = '<strong>%s</strong>:' % html_escape(
            parsed_feed.feed.get('title', 'n/a'))
        text = [title]
        xhtml = [xtitle]
        feed_id = parsed_feed.feed.get('id', feed)
        if not self.seen.setdefault(feed_id):
            # Fills with post id when first started (prevent from posting all
            # entries at startup)
            self.seen[feed_id] = {p.id for p in parsed_feed.entries}
            return

        # Detecting new post
        entries = {p.id for p in parsed_feed.entries}
        new_entries = [p for p in parsed_feed.entries
                       if p.id in entries - self.seen.get(feed_id)]
        for post in new_entries:
            self.plugin.log.info(post.title)

            body = '%(title)s %(link)s' % post
            text.append(body)

            xpost = dict(**post)
            xpost['title'] = html_escape(xpost.get('title', 'n/a'))
            xbody = '<a href="%(link)s">%(title)s</a>' % xpost
            xhtml.append(xbody)
        # Updating self.seen
        self.seen[feed_id] = entries
        if len(text) > 1:
            self.plugin.send({'mbody':'<br />'.join(xhtml), 'mhtml':'\n'.join(text)})

    def run(self):
        while not self.thread_killed:
            self.plugin.log.debug('feeds check')
            for feed in self.feeds_list:
                try:
                    self.new_posts(feed)
                except Exception as err:
                    self.plugin.log.error('feeds thread crashed: %s' % err)
                    self.plugin.log.error(''.join(traceback.format_exc()))
                    self.thread_killed = True
            self.last_check = datetime.datetime.utcnow()
            for _ in list(range(self.tempo)):
                time.sleep(1)
                if self.thread_killed:
                    return


class Feeds(Plugin):
    TEMPO = 60
    FEEDS = [
        # not working <http://bugs.debian.org/612274>
        # 'http://www.debian.org/security/dsa',

        # not working <http://bugs.debian.org/612274>
        # 'http://www.debian.org/News/news',

        # DPN in french
       'http://www.debian.org/News/weekly/dwn.fr.rdf',

        # Misc
       'http://rss.gmane.org/topics/excerpts/gmane.linux.debian.devel.announce',
       'http://rss.gmane.org/gmane.linux.debian.user.security.announce',
       'http://planet-fr.debian.net/users/rss20.xml',
       'http://planet.debian.org/atom.xml',
        ]

    def __init__(self, bot):
        Plugin.__init__(self, bot)
        self.last_check = None
        self.th_mon = FeedMonitor(self)
        self.th_mon.start()

    def shutdown(self):
        self.th_mon.thread_killed = True

    @botcmd
    def feeds(self, message, args):
        """feeds monitors debian project related feeds.
        !feeds : registred feeds list
        !feeds last : last check time"""
        if 'last' in args:
            self.send('Last feeds check: %s' % self.th_mon.last_check)
            return
        html = ['<a href="{0}">{1}</a>'.format(u, u[7:]) for u in Feeds.FEEDS]
        msg = {
                'mbody': 'Feeds:\n' + '\n'.join(Feeds.FEEDS),
                'mhtml': 'Feeds:<br />' + '<br />'.join(html)
                }
        self.send(msg)
